package ftcomputing.roboTX;

//-------------------------------------------------------------------------------
//
// FishFaceTX.java                                      Ulrich Mller    26.11.09
//                                                      Aspire Eclipse 3.4 Java 6
// Basisklasse zum Betrieb des fischertechnik ROBO TX Controller
//     Basis ist javaFish50.DLLL - umFish50.DLL - ftMscLib.DLL 
//     zugehrende Klasse JavaFish, von der die Klasse FishFace abgeleitet ist.
//     Der Funktionsinhalt orientiert sich an der Klasse FishFace der 
//     C# Assembly FishFaceTX.DLL
//     Dazu gehren noch die enums : Cnt, Ctr, Dir, Mot, Out und Unv
//
// -------------------------------------------------------------------------------

public class FishFaceTX extends JavaFish {
	
	private final int jxError        = 0xE0000001;
	private final int jxPollInterval = 12;

	private boolean notHalt = true;
	private boolean FOpen   = true;
	public FishFaceTX() {
		super();
	}
	
	public static String Version() {
		return "5.0.3.0";
	}
	public boolean getNotHalt() {
		return notHalt;
	}
	public void setNotHalt(boolean OnOff) {
		notHalt = OnOff;
	}
	
	public void openController(String ComName){
	  int ComNr = Integer.parseInt(ComName.substring(3));
		if (jxOpenController(ComNr) == jxError)
			throw new FishFaceException("ControllerProblem.OpenController");
	  notHalt=false;
      FOpen = true;
	}
	public void closeController() {
	  if (!FOpen) {
		if (jxCloseController() == jxError) { FOpen = false; notHalt = true;
			throw (new FishFaceException("ControllerProblem.CloseController")); }
	  }
    }
	
	public boolean finish() {
		return finish(null);
	}
	public boolean finish(Unv inpNr) {
		Thread.yield();
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.Finish"));
		if (notHalt || escape() != 0)
			return true;
		try {
		  if (inpNr != null && getInput(inpNr))
			return true;			
		}
		catch (FishFaceException e) {
		  FOpen = false;
		  throw (new FishFaceException("ControllerProblem.Finish"));
		}
		return false;
	 }
	public void pause(int mSek) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.Pause"));
		int sZeit = getTickCount();
		do {
			Thread.yield();
			if (notHalt || escape() != 0)
				return;
			try{Thread.sleep(jxPollInterval);}catch(InterruptedException ei){}
		} while (getTickCount() - sZeit < mSek);
	}

	public int getAnalog(Unv inpNr) {
		return getAnalog(Ctr.Main, inpNr);
	}
	public int getAnalog(Ctr ctrId, Unv inpNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getAnalog"));
		int res = jxGetAnalog(ctrId.ordinal(), inpNr.ordinal());
		if (res == jxError) { FOpen = false;
			throw (new FishFaceException("ControllerProblem.getAnalog")); }
		return res;
	}
	public int getDistance(Unv inpNr) {
		return getDistance(Ctr.Main, inpNr);
	}
	public int getDistance(Ctr ctrId, Unv inpNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getDistance"));
		int res = jxGetDistance(ctrId.ordinal(), inpNr.ordinal());
		if (res == jxError) { FOpen = false;
			throw (new FishFaceException("ControllerProblem.getDistance"));	}
		return res;		
	}
	public boolean getInput(Unv inpNr) {
		return getInput(Ctr.Main, inpNr);
	}
	public boolean getInput(Ctr ctrId, Unv inpNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getInput"));
		int res = jxGetInput(ctrId.ordinal(), inpNr.ordinal());
		if (res == jxError) { FOpen = false;
			throw (new FishFaceException("ControllerProblem.getInput")); }
		return (res == 1) ? true : false;
	}
	public int getTrack(Unv inpNr) {
		return getTrack(Ctr.Main, inpNr);
	}
	public int getTrack(Ctr ctrId, Unv inpNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getTrack"));
		int res = jxGetTrack(ctrId.ordinal(), inpNr.ordinal());
		if (res == jxError) { FOpen = false;
			throw (new FishFaceException("ControllerProblem.GetTrack")); }
		return res;
	}
	public int getVoltage(Unv inpNr) {
		return getVoltage(Ctr.Main, inpNr);
	}
	public int getVoltage(Ctr ctrId, Unv inpNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getVoltage"));
		int res = jxGetVoltage(ctrId.ordinal(), inpNr.ordinal());
		if (res == jxError) { FOpen = false;
			throw (new FishFaceException("ControllerProblem.getVoltage")); }
		return res;
	}

	public int getCounter(Cnt cntNr) {
		return getCounter(Ctr.Main, cntNr);
	}
	public int getCounter(Ctr ctrId, Cnt cntNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.getCounter"));
		int res = jxGetCounter(ctrId.ordinal(), cntNr.ordinal());
		if (res == jxError) {FOpen = false;
			throw (new FishFaceException("ControllerProblem.getCounter")); }
		return res;
	}
	public int clearCounter(Cnt cntNr) {
		return clearCounter(Ctr.Main, cntNr);
	}
	public int clearCounter(Ctr ctrId, Cnt cntNr) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.clearCounter"));
		int res = jxClearCounter(ctrId.ordinal(), cntNr.ordinal());
		if (res == jxError) {FOpen = false;
			throw (new FishFaceException("ControllerProblem.clearCounter")); }
		return res;
	}

	public void setLamp(Out outNr, int OnOff) {
		setLamp(Ctr.Main, outNr, OnOff);
	}
	public void setLamp(Ctr ctrId, Out outNr, int OnOff) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.setLamp"));
		if (jxSetLamp(ctrId.ordinal(), outNr.ordinal(), OnOff) == jxError) { 
			FOpen = false;
			throw (new FishFaceException("ControllerProblem.setLamp")); }
	}

	public void setMotor(Mot motNr, Dir direction) {
		setMotor(Ctr.Main, motNr, direction);
	}
	public void setMotor(Ctr ctrId, Mot motNr, Dir direction) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.setMotor"));
		if (jxSetMotor(ctrId.ordinal(), motNr.ordinal(), direction.ordinal(), 512)
				== jxError) {FOpen = false;
			throw (new FishFaceException("ControllerProblem.setMotor"));}
	}
	public void speedMotor(Mot motNr, Dir direction, int speed) {
		speedMotor(Ctr.Main, motNr, direction, speed);
	}
	public void speedMotor(Ctr ctrId, Mot motNr, Dir direction, int speed) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.setSpeedMotor"));
		if (jxSetMotor(ctrId.ordinal(), motNr.ordinal(), direction.ordinal(), speed)
				== jxError){
			FOpen = false;
			throw (new FishFaceException("ControllerProblem.setSpeedMotor")); }
	}
	public void startMotor(Mot motNr, Dir direction, int speed, int iCount) {
		startMotor(Ctr.Main, motNr, direction, speed, iCount);
	}
	public void startMotor(Ctr ctrId, Mot motNr, Dir direction, 
			int speed, int iCount) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.setStartMotor"));
		if (jxStartMotor(ctrId.ordinal(), motNr.ordinal(), direction.ordinal(), 
				speed, iCount, 0) == jxError){
			FOpen = false;
			throw (new FishFaceException("ControllerProblem.setStartMotor")); }
	}
	public void startRobMotor(Mot motNr, Dir direction, int speed, int iCount) {
		startRobMotor(Ctr.Main, motNr, direction, speed, iCount);
	}
	public void startRobMotor(Ctr ctrId, Mot motNr, Dir direction, 
			int speed, int iCount) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.setStartRobMotor"));
		if (jxStartMotor(ctrId.ordinal(), motNr.ordinal(), direction.ordinal(), 
				speed, iCount, 1) == jxError){
			FOpen = false;
			throw (new FishFaceException("ControllerProblem.setStartRobMotor")); }
	}

	public int waitForMotor(Mot motNr) {
		return waitForMotor(Ctr.Main, motNr);
	}
	public int waitForMotor(Ctr ctrId, Mot motNr) {
		if(!FOpen)
			throw (new FishFaceException("KeinOpen.waitForMotor"));
		int res = 0;
		do {
			res = jxIsMotorReady(ctrId.ordinal(), motNr.ordinal());
			if (res == jxError) {
				FOpen = false;
				throw (new FishFaceException("ControllerProblem.waitForMotor"));
			}
		} while (res < 0 && !notHalt && escape() == 0);
		return res;
	}
	
	public void waitForMotors(Mot... motorNrs) {
		waitForMotors(Ctr.Main, motorNrs);
	}
	public void waitForMotors(Ctr ctrId, Mot... motorNrs) {
		if(!FOpen)
			throw (new FishFaceException("KeinOpen.waitForMotors"));
		int c = 0;
		int res;
		do {
			c = 0;
			for (Mot motNr : motorNrs) {
				res = jxIsMotorReady(ctrId.ordinal(), motNr.ordinal());
				if (res == jxError) { FOpen = false;
				throw (new FishFaceException("ControllerProblem.waitForMotors"));
				}
				c += res < 0 ? 1 : 0; 
			}
			if (c == 0) return;
		} while (!notHalt && escape() == 0);
	}
	
	public void waitForInput(Unv inpNr, boolean OnOff) {
		waitForInput(Ctr.Main, inpNr, OnOff);
	}
	public void waitForInput(Ctr ctrId, Unv inpNr, boolean OnOff) {
		if (!FOpen)
			throw (new FishFaceException("KeinOpen.waitForInput"));
		do {
			if (notHalt || escape() != 0) return;
			try{Thread.sleep(jxPollInterval);}catch(InterruptedException ei){}
		} while (!getInput(inpNr) == OnOff);
	}
	public void waitForHigh(Unv inpNr) {
		waitForHigh(Ctr.Main, inpNr);
	}
	public void waitForHigh(Ctr ctrId, Unv inpNr) {
		waitForInput(ctrId, inpNr, false);
		waitForInput(ctrId, inpNr, true);
	}
	public void waitForLow(Unv inpNr) {
		waitForLow(Ctr.Main, inpNr);
	}
	public void waitForLow(Ctr ctrId, Unv inpNr) {
		waitForInput(ctrId, inpNr, true);
		waitForInput(ctrId, inpNr, false);
	}
	
}
